<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://orionorigin.com
 * @since      1.0.0
 *
 * @package    Vpc_Rqa
 * @subpackage Vpc_Rqa/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Vpc_Rqa
 * @subpackage Vpc_Rqa/public
 * @author     orion <help@orionorigin.com>
 */
class Vpc_Rqa_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string $plugin_name       The name of the plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		wp_enqueue_style( 'validationEngine.jquery', plugin_dir_url( __FILE__ ) . 'css/validationEngine.jquery.css', array(), VPC_RQA_VERSION, 'all' );
		wp_enqueue_style( 'vpc-rqa-public-css', plugin_dir_url( __FILE__ ) . 'css/vpc-rqa-public.css', array(), VPC_RQA_VERSION, 'all' );
		wp_enqueue_style( 'o-bs-modal-css', VPC_URL . 'admin/js/modal/modal.min.css', array(), VPC_VERSION, 'all' );
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		wp_enqueue_script( 'jquery.validationEngine', plugin_dir_url( __FILE__ ) . 'js/languages/jquery.validationEngine.js', array( 'jquery' ), '2.6.2', false );
		$lang = get_locale();
		switch ( $lang ) {
			case 'fr_FR':
				wp_enqueue_script( 'jquery.validationEngine-fr', plugin_dir_url( __FILE__ ) . 'js/languages/jquery.validationEngine-fr.js', array( 'jquery' ), '2.6.2', false );
				break;
			case 'it_IT':
				wp_enqueue_script( 'jquery.validationEngine-it', plugin_dir_url( __FILE__ ) . 'js/languages/jquery.validationEngine-it.js', array( 'jquery' ), '2.6.2', false );
				break;
			case 'en_EN':
				wp_enqueue_script( 'jquery.validationEngine-en', plugin_dir_url( __FILE__ ) . 'js/languages/jquery.validationEngine-en.js', array( 'jquery' ), '2.6.2', false );
				break;
			case 'de_DE':
				wp_enqueue_script( 'jquery.validationEngine-de', plugin_dir_url( __FILE__ ) . 'js/languages/jquery.validationEngine-de.js', array( 'jquery' ), '2.6.2', false );
				break;
			case 'uk_UK':
				wp_enqueue_script( 'jquery.validationEngine-uk', plugin_dir_url( __FILE__ ) . 'js/languages/jquery.validationEngine-uk.js', array( 'jquery' ), '2.6.2', false );
				break;
			case 'es_ES':
				wp_enqueue_script( 'jquery.validationEngine-es', plugin_dir_url( __FILE__ ) . 'js/languages/jquery.validationEngine-es.js', array( 'jquery' ), '2.6.2', false );
				break;
			default:
				wp_enqueue_script( 'jquery.validationEngine-en', plugin_dir_url( __FILE__ ) . 'js/languages/jquery.validationEngine-en.js', array( 'jquery' ), '2.6.2', false );
				break;
		}
		wp_enqueue_script( 'o-modal-js', VPC_URL . 'admin/js/modal/modal.min.js', array( 'jquery' ), VPC_VERSION, false );
		wp_enqueue_script( 'vpc-rqa-public-js', VPC_RQA_URL . 'public/js/vpc-rqa-public.js', array( 'jquery' ), VPC_RQA_VERSION, false );
	}

	/**
	 * Function to add buttons to the configurator.
	 *
	 * @param  array  $buttons Buttons list.
	 * @param  string $prod_id Product ID.
	 * @return array
	 */
	public function add_buttons( $buttons, $prod_id ) {
		$config = $this->get_config_data( $prod_id );
		if ( isset( $config['request-quote'] ) && 'Yes' === $config['request-quote'] ) {
			$request_btn = array(
				'id'         => 'vpc-request-quote',
				'label'      => __( 'Request a quote', 'vpc-rqa' ),
				'class'      => 'o-modal-trigger',
				'attributes' => array(
					'data-toggle'  => 'o-modal',
					'data-prod-id' => $prod_id,
					'data-modalid' => 'raq-modal',
				),
			);

			$request_btn = apply_filters( 'vpc_request_quote_btn', $request_btn, $prod_id );
			array_push( $buttons, $request_btn );
			if ( 'No' === get_proper_value( $config, 'add-to-cart', 'No' ) ) {
				?>
		<style>
			#vpc-add-to-cart{
			display:none;
			}
		</style>
				<?php

			}
		}
		return $buttons;
	}

	/**
	 * Function to display request a qote modal form.
	 *
	 * @param  array  $config Configuration's datas.
	 * @param  string $product_id Product ID.
	 * @param  string $config_id Config ID.
	 */
	public function add_modal_html_bloc( $config, $product_id, $config_id ) {
		$html = $this->build_form( $product_id );
		echo '<div class="omodal fade o-modal" id="raq-modal" role="dialog" aria-labelledby="myModalLabel" aria-hidden="true">
		<div class="omodal-content">
		<div class="omodal-header wpc_header">
		<button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>
		<div class="omodal-title rqa_wpc_header" id="myModalLabel">' . esc_html__( 'REQUEST A QUOTE', 'vpc-rqa' ) . '</div>
		</div>
		' . wp_kses( $html, get_allowed_tags() ) . '
		</div>
		</div>';
	}

	/**
	 * Function to get the configuration's data.
	 *
	 * @param  string $prod_id Product ID.
	 * @return array           Configuration's data.
	 */
	private function get_config_data( $prod_id ) {
		$ids         = get_product_root_and_variations_ids( $prod_id );
		$config_meta = get_post_meta( $ids['product-id'], 'vpc-config', true );
		$configs     = get_proper_value( $config_meta, $prod_id, array() );
		$config_id   = get_proper_value( $configs, 'config-id', false );
		$config      = get_post_meta( $config_id, 'vpc-config', true );
		return $config;
	}

	/**
	 * Function that gather messages that will be used as notice information in this addon.
	 *
	 * @param  array $datas List of datas.
	 * @return array        List of datas.
	 */
	public function add_data( $datas ) {
		$datas['email_sent']      = __( 'Your quote request has been sent. <br> We will be in touch soon. Thank you for your business', 'vpc-rqa' );
		$datas['name_empty']      = __( 'Name field is empty', 'vpc-rqa' );
		$datas['rqa_modal_title'] = __( 'REQUEST A QUOTE', 'vpc-rqa' );
		$datas['company_empty']   = __( 'Company field is empty', 'vpc-rqa' );
		$datas['email_empty']     = __( 'Email field is empty', 'vpc-rqa' );
		$datas['phone_empty']     = __( 'Phone field is empty', 'vpc-rqa' );
		$datas['email_invalid']   = __( 'Invalid Email provided. Please check and try again!', 'vpc-rqa' );
		$datas['progess_msg']     = __( 'Sending in progress....!', 'vpc-rqa' );
		$datas['form']            = $this->build_form( $datas['product'] );
		$datas['loading_url']     = plugin_dir_url( __FILE__ ) . 'images/loading.gif';
		return $datas;
	}

	/**
	 * Function to create the default form.
	 *
	 * @return string             The default form in html code.
	 */
	private function create_default_form() {
		$html = '
		<div class="modal-content">
		<div class="error"></div>
		<div class="line">
		<div>
		<input id="name" name="name" data-validation-engine="validate[required]" type="text" value="" placeholder="' . __( 'Name', 'vpc-rqa' ) . '"/>
		</div>
		</div>
		<div class="line">
		<div>
		<input id="company" name="company" data-validation-engine="validate[required]" type="text" value="" placeholder="' . __( 'Company', 'vpc-rqa' ) . '" />
		</div>
		</div>
		<div class="line">
		<div>
		<input id="email"  name="email" data-validation-engine="validate[required,custom[email]]" type="text" value="" placeholder="' . __( 'Email Address', 'vpc-rqa' ) . '"/>
		</div>
		</div>
		<div class="line">
		<div>
		<input id="phone" name="phone" data-validation-engine="validate[required]" type="text" value="" placeholder="' . __( 'Phone', 'vpc-rqa' ) . '"/>
		</div>
		</div>

		</div>';

		$html = apply_filters( 'vpc_rqa_create_default_form', $html );
		return $html;
	}

	/**
	 * Function to save the form datas.
	 */
	public function save_form_datas() {
		wp_verify_nonce( 'save_form_datas', 'rqa_form_datas_nonce' );
		$datas         = '';
		$recap         = array();
		$prod_id       = '';
		$multiview     = '';
		$file_path     = '';
		$preview_saved = array();
		$quantity      = 1;
		if ( isset( $_POST['frm_data'] ) ) {
			$datas = vpc_rqa_sanitize_array( wp_unslash( $_POST['frm_data'] ) ); // phpcs:ignore
		}
		if ( isset( $_POST['views'] ) ) {
			$views = vpc_rqa_sanitize_array( wp_unslash( $_POST['views'] ) ); // phpcs:ignore
		}
		/*if ( isset( $_POST['recap'] ) ) {
			$recap = wp_unslash( $_POST['recap'] ); // phpcs:ignore
		}*/
		if ( isset( $_POST['recap'] ) ) {
			$sanitize_recap = vpc_rqa_sanitize_array( wp_unslash( $_POST['recap'] ) ); // phpcs:ignore
			$recap          = vpc_rqa_clean_recap( $sanitize_recap );
		}
		if ( isset( $_POST['prod_id'] ) && ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['prod_id'] ) ) ) ) {
			$prod_id = sanitize_text_field( wp_unslash( $_POST['prod_id'] ) );
		}
		if ( isset( $_POST['quantity'] ) && ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['quantity'] ) ) ) ) {
			$quantity = sanitize_text_field( wp_unslash( $_POST['quantity'] ) );
		}
		if ( isset( $_POST['multiview'] ) && ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['multiview'] ) ) ) ) {
			$multiview = sanitize_text_field( wp_unslash( $_POST['multiview'] ) );
		}
		if ( isset( $_POST['preview_saved'] ) && ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['preview_saved'] ) ) ) ) {
			$previews = sanitize_text_field( wp_unslash( $_POST['preview_saved'] ) );
			$previews = json_decode( stripslashes( $previews ) );
			if ( class_exists( 'Vpc_Mva' ) && 'true' === $multiview ) {
				foreach ( $previews as $view => $url ) {
					$preview_saved[ $views[ $view ] ] = $url;
				}
			} else {
				$preview_saved = $previews;
			}
		}
		
		$result = $this->sent_email_with_formbuild_datas( $preview_saved, $recap, $datas, $prod_id, $multiview, $file_path, $quantity );
		$this->add_custom_post( $datas, $quantity, $recap, $preview_saved, $multiview, $prod_id );
		do_action( 'vpc_rqa_after_mail_sent', $preview_saved, $recap, $datas, $prod_id, $multiview, $file_path );
		die();
	}

	/**
	 * Function to get the text's html.
	 *
	 * @param  array  $img_merged Merged images.
	 * @param  string $multiview  Mutliview state value.
	 * @param  array  $config     Configuration's data.
	 * @return string                    The text's html.
	 */
	public function get_text_html( $img_merged, $multiview, $config ) {
		$text_html   = '';
		$vpc_options = get_option( 'vpc-options' );
		if ( is_array( $img_merged ) ) {
			foreach ( $img_merged as $key => $value ) {
				if ( isset( $multiview ) && 'true' === $multiview ) {
					if ( isset( $vpc_options['rqa-image-height'] ) && $vpc_options['rqa-image-height'] > 0 && isset( $vpc_options['rqa-image-width'] ) && $vpc_options['rqa-image-width'] > 0 ) {
						$text_html .= "<div style='font-weight:bold;font-size:12px'> View " . $key . "</div><img style='width:100%;' src='" . aq_resize( $value, intval($vpc_options['rqa-image-width']),intval($vpc_options['rqa-image-height']), true ) . "' /><br>";
					} else {
						$text_html .= "<div style='font-weight:bold;font-size:12px'> View " . $key . "</div><img style='width:100%;' src='" . $value . "' /><br>";
					}
				} elseif ( isset( $multiview ) && 'false' === $multiview ) {
					if ( isset( $vpc_options['rqa-image-height'] ) && $vpc_options['rqa-image-height'] > 0 && isset( $vpc_options['rqa-image-width'] ) && $vpc_options['rqa-image-width'] > 0 ) {
						$text_html .= "<div><img style='width:60%' src='" . aq_resize( $value, intval($vpc_options['rqa-image-width']), intval($vpc_options['rqa-image-height']), true ) . "' /></div><br>";
					} else {
						$text_html .= "<div><img style='width:60%' src='" . $value . "' /></div><br>";
					}
				}
			}
		}
		$text_html = apply_filters( 'vpc_rqa_get_text_html', $text_html, $img_merged, $multiview, $config );
		return $text_html;
	}

	/**
	 * Function to get the mail headers.
	 *
	 * @param  array $datas       Datas list.
	 * @param  array $vpc_options The VPC options.
	 * @param  array $config      Configuration's datas.
	 * @return array              The headers datas.
	 */
	public function get_mail_headers( $datas, $vpc_options, $config ) {
		$headers = array();
		if ( get_proper_value( $config, 'send-user-mail', 'Yes' ) === 'Yes' ) {
			if ( isset( $datas['email'] ) && filter_var( $datas['email'], FILTER_VALIDATE_EMAIL ) ) {
				$headers[] = 'Cc:' . $datas['email'];
			}
			foreach ( $datas as $key => $value ) {
				if ( filter_var( $value, FILTER_VALIDATE_EMAIL ) && ! in_array( $value, $headers, true ) ) {
					$headers[] = 'Cc:' . $value;
				}
			}
		}
		$headers = apply_filters( 'vpc_rqa_headers', $headers, $datas, $vpc_options, $config );
		return $headers;
	}

	/**
	 * Function to get the mail sender's name.
	 *
	 * @param  string $from The mail sender's data.
	 * @return string       The mail sender's name.
	 */
	public function get_mail_sender_name( $from ) {
		$vpc_options = get_option( 'vpc-options' );

		$sender_name = get_proper_value( $vpc_options, 'sender_name', 'WordPress' );
		$sender_name = apply_filters( 'vpc_rqa_get_mail_sender_name', $sender_name, $from, $vpc_options );
		return $sender_name;
	}

	/**
	 * Function to get the mail message.
	 *
	 * @param  array  $user_datas            User datas.
	 * @param  array  $configuration_options Options of a configuration.
	 * @param  string $text_html             The text's html.
	 * @param  float  $total_price           The total price.
	 * @param  array  $config                Configuration's datas.
	 * @param  string $prod_id               Product ID.
	 * @return string                        The mail's message.
	 */
	public function get_mail_message( $user_datas, $configuration_options, $text_html, $total_price, $config, $prod_id ) {
		$vpc_settings = get_option( 'vpc-options' );
		$config_data  = $this->get_config_data( $prod_id );
		// @codingStandardsIgnoreStart
		if ( isset( $config_data ) && ! empty( $config_data ) && get_proper_value( $config_data, 'request-quote', 'Yes' ) === 'Yes' && get_proper_value( $config_data, 'show-price-in-mail', 'Yes' ) === 'No' ) {
			$message = '<!DOCTYPE html><html xmlns="http://www.w3.org/1999/xhtml" xmlns:v="urn:schemas-microsoft-com:vml" xmlns:o="urn:schemas-microsoft-com:office:office"><head>  <title></title>  <!--[if !mso]><!-- -->  <meta http-equiv="X-UA-Compatible" content="IE=edge">  <!--<![endif]--><meta http-equiv="Content-Type" content="text/html; charset=UTF-8"><meta name="viewport" content="width=device-width, initial-scale=1.0"><style type="text/css">  #outlook a { padding: 0; }  .ReadMsgBody { width: 100%; }  .ExternalClass { width: 100%; }  .ExternalClass * { line-height:100%; }  body { margin: 0; padding: 0; -webkit-text-size-adjust: 100%; -ms-text-size-adjust: 100%; }  table, td { border-collapse:collapse; mso-table-lspace: 0pt; mso-table-rspace: 0pt; }  img { border: 0; height: auto; line-height: 100%; outline: none; text-decoration: none; -ms-interpolation-mode: bicubic; width: 100%; }  p { display: block; margin: 13px 0; }</style><!--[if !mso]><!--><style type="text/css">  @media only screen and (max-width:480px) {    @-ms-viewport { width:320px; }    @viewport { width:320px; }  }</style><!--<![endif]--><!--[if mso]><xml>  <o:OfficeDocumentSettings>    <o:AllowPNG/>    <o:PixelsPerInch>96</o:PixelsPerInch>  </o:OfficeDocumentSettings></xml><![endif]--><!--[if lte mso 11]><style type="text/css">  .outlook-group-fix {    width:100% !important;  }</style><![endif]--><!--[if !mso]><!-->    <link href="https://fonts.googleapis.com/css?family=Ubuntu:300,400,500,700" rel="stylesheet" type="text/css">    <style type="text/css">        @import url("https://fonts.googleapis.com/css?family=Ubuntu:300,400,500,700");    </style>  <!--<![endif]--><style type="text/css">  @media only screen and (min-width:480px) {    .mj-column-per-100 { width:100%!important; }.mj-column-per-50 { width:50%!important; }  }</style></head><body style="background: #FFFFFF;">    <div class="mj-container" style="background-color:#FFFFFF;"><!--[if mso | IE]>      <table role="presentation" border="0" cellpadding="0" cellspacing="0" align="center" style="width:600px;">        <tr>          <td style="line-height:0px;font-size:0px;mso-line-height-rule:exactly;">      <![endif]--><table role="presentation" cellpadding="0" cellspacing="0" style="font-size:0px;width:100%;" border="0"><tbody><tr><td><div style="margin:0px auto;max-width:500px;"><table role="presentation" cellpadding="0" cellspacing="0" style="font-size:0px;width:100%;" align="center" border="0"><tbody><tr><td style="text-align:center;vertical-align:top;direction:ltr;font-size:0px;padding:9px 0px 9px 0px;"><!--[if mso | IE]>      <table role="presentation" border="0" cellpadding="0" cellspacing="0">        <tr>          <td style="vertical-align:top;width:600px;">      <![endif]--><div class="mj-column-per-100 outlook-group-fix" style="vertical-align:top;display:inline-block;direction:ltr;font-size:13px;text-align:left;width:100%;"><table role="presentation" cellpadding="0" cellspacing="0" width="100%" border="0"><tbody><tr><td style="word-wrap:break-word;font-size:0px;padding:0px 20px 0px 20px;" align="center"><div style="cursor:auto;color:#000000;font-family:Ubuntu, Helvetica, Arial, sans-serif;font-size:11px;line-height:22px;text-align:center;font-weight: bold;"><p>' . __( 'REQUESTED CONFIGURATION', 'vpc-rqa' ) . '</p></div></td></tr></tbody></table></div><!--[if mso | IE]>      </td></tr></table>      <![endif]--></td></tr></tbody></table></div></td></tr></tbody></table><!--[if mso | IE]>      </td></tr></table>      <![endif]-->      <!--[if mso | IE]>      <table role="presentation" border="0" cellpadding="0" cellspacing="0" width="600" align="center" style="width:600px;">        <tr>          <td style="line-height:0px;font-size:0px;mso-line-height-rule:exactly;">      <![endif]--><div style="margin:0px auto;min-width:600px;"><table role="presentation" cellpadding="0" cellspacing="0" style="font-size:0px;width:100%;" align="center" border="0"><tbody><tr><td style="text-align:center;vertical-align:top;direction:ltr;font-size:0px;padding:20px 0px 20px 0px;"><!--[if mso | IE]>      <table role="presentation" border="0" cellpadding="0" cellspacing="0">        <tr>          <td style="vertical-align:top;width:300px;">      <![endif]--><div class="mj-column-per-50 outlook-group-fix" style="vertical-align:top;display:inline-block;direction:ltr;font-size:13px;text-align:left;width:100%;"><table role="presentation" cellpadding="0" cellspacing="0" width="100%" border="0"><tbody><tr><td style="word-wrap:break-word;font-size:0px;padding:0px 0px 0px 0px;" align="center"><table role="presentation" cellpadding="0" cellspacing="0" style="border-collapse:collapse;border-spacing:0px;" align="center" border="0"><tbody><tr><td style=""><div style="font-size:14px;font-weight: bold;padding: 10px 0; text-align: center;">' . __( 'Image generated :', 'vpc-rqa' ) . '</div><div>' . $text_html . '</div></td></tr></tbody></table></td></tr></tbody></table></div><!--[if mso | IE]>      </td><td style="vertical-align:top;width:300px;">      <![endif]--><div class="mj-column-per-50 outlook-group-fix" style="vertical-align:top;display:inline-block;direction:ltr;font-size:13px;text-align:left;width:100%;"><table role="presentation" cellpadding="0" cellspacing="0" width="100%" border="0"><tbody><tr><td style="word-wrap:break-word;font-size:0px;" align="left"><div style="cursor:auto;color:#000000;font-family:Ubuntu, Helvetica, Arial, sans-serif;text-align:left;font-size:14px; padding: 0 20px;"><p style="font-weight:bold;">' . __( 'User Info :', 'vpc-rqa' ) . '</p><div style="font-size:14px; padding: 0 10px;">' . $user_datas . '</div><br></div></td></tr><tr><td style="word-wrap:break-word;font-size:0px;padding:10px 0px;"><p style="font-size:1px;margin:0px;border-top:1px solid #000;width:100%;"></p><!--[if mso | IE]><table role="presentation" align="center" border="0" cellpadding="0" cellspacing="0" style="font-size:1px;margin:0px auto;border-top:1px solid #000;width:100%;" width="300"><tr><td style="height:0;line-height:0;"> </td></tr></table><![endif]--></td></tr><tr><td style="word-wrap:break-word;font-size:0px;" align="left"><div style="cursor:auto;color:#000000;font-family:Ubuntu, Helvetica, Arial, sans-serif;font-size:11px;text-align:left;font-size:14px; padding: 0 20px;"><p style="font-weight:bold;">' . __( 'Configuration details :', 'vpc-rqa' ) . '</p><div style="font-size:14px; padding: 0 10px;">' . $configuration_options . '</div><br></div></td></tr><tr><td style="word-wrap:break-word;font-size:0px;padding:10px 0px;"><p style="font-size:1px;margin:0px;border-top:1px solid #000;width:100%;"></p><!--[if mso | IE]><table role="presentation" align="center" border="0" cellpadding="0" cellspacing="0" style="font-size:1px;margin:0px auto;border-top:1px solid #000;width:100%;" width="300"><tr><td style="height:0;line-height:0;"> </td></tr></table><![endif]--></td></tr></tbody></table></div><!--[if mso | IE]>      </td></tr></table>      <![endif]--></td></tr></tr></tbody></table></div><!--[if mso | IE]>      </td></tr></table>      <![endif]--></div></body></html>';
		} else {
			$message = '<!DOCTYPE html><html xmlns="http://www.w3.org/1999/xhtml" xmlns:v="urn:schemas-microsoft-com:vml" xmlns:o="urn:schemas-microsoft-com:office:office"><head>  <title></title>  <!--[if !mso]><!-- -->  <meta http-equiv="X-UA-Compatible" content="IE=edge">  <!--<![endif]--><meta http-equiv="Content-Type" content="text/html; charset=UTF-8"><meta name="viewport" content="width=device-width, initial-scale=1.0"><style type="text/css">  #outlook a { padding: 0; }  .ReadMsgBody { width: 100%; }  .ExternalClass { width: 100%; }  .ExternalClass * { line-height:100%; }  body { margin: 0; padding: 0; -webkit-text-size-adjust: 100%; -ms-text-size-adjust: 100%; }  table, td { border-collapse:collapse; mso-table-lspace: 0pt; mso-table-rspace: 0pt; }  img { border: 0; height: auto; line-height: 100%; outline: none; text-decoration: none; -ms-interpolation-mode: bicubic; width: 100%; }  p { display: block; margin: 13px 0; }</style><!--[if !mso]><!--><style type="text/css">  @media only screen and (max-width:480px) {    @-ms-viewport { width:320px; }    @viewport { width:320px; }  }</style><!--<![endif]--><!--[if mso]><xml>  <o:OfficeDocumentSettings>    <o:AllowPNG/>    <o:PixelsPerInch>96</o:PixelsPerInch>  </o:OfficeDocumentSettings></xml><![endif]--><!--[if lte mso 11]><style type="text/css">  .outlook-group-fix {    width:100% !important;  }</style><![endif]--><!--[if !mso]><!-->    <link href="https://fonts.googleapis.com/css?family=Ubuntu:300,400,500,700" rel="stylesheet" type="text/css">    <style type="text/css">        @import url("https://fonts.googleapis.com/css?family=Ubuntu:300,400,500,700");    </style>  <!--<![endif]--><style type="text/css">  @media only screen and (min-width:480px) {    .mj-column-per-100 { width:100%!important; }.mj-column-per-50 { width:50%!important; }  }</style></head><body style="background: #FFFFFF;">    <div class="mj-container" style="background-color:#FFFFFF;"><!--[if mso | IE]>      <table role="presentation" border="0" cellpadding="0" cellspacing="0" align="center" style="width:600px;">        <tr>          <td style="line-height:0px;font-size:0px;mso-line-height-rule:exactly;">      <![endif]--><table role="presentation" cellpadding="0" cellspacing="0" style="font-size:0px;width:100%;" border="0"><tbody><tr><td><div style="margin:0px auto;max-width:500px;"><table role="presentation" cellpadding="0" cellspacing="0" style="font-size:0px;width:100%;" align="center" border="0"><tbody><tr><td style="text-align:center;vertical-align:top;direction:ltr;font-size:0px;padding:9px 0px 9px 0px;"><!--[if mso | IE]>      <table role="presentation" border="0" cellpadding="0" cellspacing="0">        <tr>          <td style="vertical-align:top;width:600px;">      <![endif]--><div class="mj-column-per-100 outlook-group-fix" style="vertical-align:top;display:inline-block;direction:ltr;font-size:13px;text-align:left;width:100%;"><table role="presentation" cellpadding="0" cellspacing="0" width="100%" border="0"><tbody><tr><td style="word-wrap:break-word;font-size:0px;padding:0px 20px 0px 20px;" align="center"><div style="cursor:auto;color:#000000;font-family:Ubuntu, Helvetica, Arial, sans-serif;font-size:11px;line-height:22px;text-align:center;font-weight: bold;"><p>' . __( 'REQUESTED CONFIGURATION', 'vpc-rqa' ) . '</p></div></td></tr></tbody></table></div><!--[if mso | IE]>      </td></tr></table>      <![endif]--></td></tr></tbody></table></div></td></tr></tbody></table><!--[if mso | IE]>      </td></tr></table>      <![endif]-->      <!--[if mso | IE]>      <table role="presentation" border="0" cellpadding="0" cellspacing="0" width="600" align="center" style="width:600px;">        <tr>          <td style="line-height:0px;font-size:0px;mso-line-height-rule:exactly;">      <![endif]--><div style="margin:0px auto;min-width:600px;"><table role="presentation" cellpadding="0" cellspacing="0" style="font-size:0px;width:100%;" align="center" border="0"><tbody><tr><td style="text-align:center;vertical-align:top;direction:ltr;font-size:0px;padding:20px 0px 20px 0px;"><!--[if mso | IE]>      <table role="presentation" border="0" cellpadding="0" cellspacing="0">        <tr>          <td style="vertical-align:top;width:300px;">      <![endif]--><div class="mj-column-per-50 outlook-group-fix" style="vertical-align:top;display:inline-block;direction:ltr;font-size:13px;text-align:left;width:100%;"><table role="presentation" cellpadding="0" cellspacing="0" width="100%" border="0"><tbody><tr><td style="word-wrap:break-word;font-size:0px;padding:0px 0px 0px 0px;" align="center"><table role="presentation" cellpadding="0" cellspacing="0" style="border-collapse:collapse;border-spacing:0px;" align="center" border="0"><tbody><tr><td style=""><div style="font-size:14px;font-weight: bold;padding: 10px 0; text-align: center;">' . __( 'Image generated :', 'vpc-rqa' ) . '</div><div>' . $text_html . '</div></td></tr></tbody></table></td></tr></tbody></table></div><!--[if mso | IE]>      </td><td style="vertical-align:top;width:300px;">      <![endif]--><div class="mj-column-per-50 outlook-group-fix" style="vertical-align:top;display:inline-block;direction:ltr;font-size:13px;text-align:left;width:100%;"><table role="presentation" cellpadding="0" cellspacing="0" width="100%" border="0"><tbody><tr><td style="word-wrap:break-word;font-size:0px;" align="left"><div style="cursor:auto;color:#000000;font-family:Ubuntu, Helvetica, Arial, sans-serif;text-align:left;font-size:14px; padding: 0 20px;"><p style="font-weight:bold;">' . __( 'User Info :', 'vpc-rqa' ) . '</p><div style="font-size:14px; padding: 0 10px;">' . $user_datas . '</div><br></div></td></tr><tr><td style="word-wrap:break-word;font-size:0px;padding:10px 0px;"><p style="font-size:1px;margin:0px;border-top:1px solid #000;width:100%;"></p><!--[if mso | IE]><table role="presentation" align="center" border="0" cellpadding="0" cellspacing="0" style="font-size:1px;margin:0px auto;border-top:1px solid #000;width:100%;" width="300"><tr><td style="height:0;line-height:0;"> </td></tr></table><![endif]--></td></tr><tr><td style="word-wrap:break-word;font-size:0px;" align="left"><div style="cursor:auto;color:#000000;font-family:Ubuntu, Helvetica, Arial, sans-serif;font-size:11px;text-align:left;font-size:14px; padding: 0 20px;"><p style="font-weight:bold;">' . __( 'Configuration details :', 'vpc-rqa' ) . '</p><div style="font-size:14px; padding: 0 10px;">' . $configuration_options . '</div><br></div></td></tr><tr><td style="word-wrap:break-word;font-size:0px;padding:10px 0px;"><p style="font-size:1px;margin:0px;border-top:1px solid #000;width:100%;"></p><!--[if mso | IE]><table role="presentation" align="center" border="0" cellpadding="0" cellspacing="0" style="font-size:1px;margin:0px auto;border-top:1px solid #000;width:100%;" width="300"><tr><td style="height:0;line-height:0;"> </td></tr></table><![endif]--></td></tr><tr><td style="word-wrap:break-word;font-size:0px;" align="left"><div style="cursor:auto;color:#000000;font-family:Ubuntu, Helvetica, Arial, sans-serif;font-size:11px;text-align:left;font-size:14px; padding: 0 20px;"><p style="font-weight:bold;">' . __( 'Total price:', 'vpc-rqa' ) . '</p><div style="font-size:14px; padding: 0 10px;">' . $total_price . '</div><br></div></td></tr></tbody></table></div><!--[if mso | IE]>      </td></tr></table>      <![endif]--></td></tr></tr></tbody></table></div><!--[if mso | IE]>      </td></tr></table>      <![endif]--></div></body></html>';
		}
		// @codingStandardsIgnoreEnd
		return $message;
	}

	/**
	 * Function to send an email with form builder's datas.
	 *
	 * @param  array  $img_merged Merged images.
	 * @param  array  $recap      A configuration resume.
	 * @param  array  $datas      Mail's datas.
	 * @param  string $prod_id    Product ID.
	 * @param  string $multiview  Multiview state value.
	 * @param  array  $files      List of files.
	 * @param  array  $quantity   Product quantity.
	 * @return bool               Whether the email contents were sent successfully.
	 */
	private function sent_email_with_formbuild_datas( $img_merged, $recap, $datas, $prod_id, $multiview, $files, $quantity ) {
		$config                = get_product_config( $prod_id );
		$configuration_options = '';
		$user_datas            = '';
		$canvases_datas        = array();
		$product_obj           = wc_get_product( $prod_id );
		if ( is_array( $datas ) ) {
			unset( $datas['id_ofb'] );
			unset( $datas['captcha'] );
			foreach ( $datas as $option => $value ) {
				if ( is_array( $value ) ) {
					$value = implode( ', ', $value );
				}
				$user_datas .= ' ' . ucfirst( $option ) . ' : ' . $value . '<br>';
			}
		}
		$prod_name             = $product_obj->get_title();
		$configuration_options = 'Product : ' . $prod_name . '<br>';
		if ( is_array( $recap ) ) {
			if ( isset( $recap['action'] ) ) {
				unset( $recap['action'] );
			}
			if ( isset( $recap['nonce'] ) ) {
				unset( $recap['nonce'] );
			}
			if ( isset( $recap['id_ofb'] ) ) {
				unset( $recap['id_ofb'] );
			}
			if ( isset( $recap['canvas_data'] ) ) {
				if ( '' !== $recap['canvas_data'] ) {
					$canvases_datas = json_decode( stripslashes( $recap['canvas_data'] ) );
				}
				unset( $recap['canvas_data'] );
			}

			foreach ( $recap as $option => $value ) {
				if ( is_array( $value ) ) {
					$value = implode( ', ', $value );
				}
				if ( strpos( $value, 'base64' ) !== false ) {
					$value = "<img style='width:10%;' src='$value' />";
				}
				$configuration_options .= ' ' . $option . ' : ' . $value . '<br>';
			}
			if ( isset( $canvases_datas ) && ! empty( $canvases_datas ) ) {
				foreach ( $canvases_datas as $canvas_key => $canvas ) {
					$configuration_options .= vpc_rqa_get_canvas_datas_recap( $canvas );
				}
			}
		}
		$configuration_options .= '<br> Quantity : ' . $quantity . '<br>';

		$total_price = $this->get_total_price( $prod_id, $recap );
		$text_html   = $this->get_text_html( $img_merged, $multiview, $config->settings );
		$vpc_options = get_option( 'vpc-options' );
		$subject     = __( 'Quote request', 'vpc-rqa' );
		$subject     = get_proper_value( $vpc_options, 'mail_title', $subject );
		$headers     = $this->get_mail_headers( $datas, $vpc_options, $config->settings );
		$recipients  = get_proper_value( $vpc_options, 'destinataire', array() );
		$message     = $this->get_mail_message( $user_datas, $configuration_options, $text_html, $total_price, $config->settings, $prod_id );
		$message     = apply_filters( 'vpc_rqa_html_sent', $message, $recap, $img_merged, $datas, $prod_id, $text_html, $total_price, $quantity );
		$message     = "<div style='width:100%'>" . $message . '</div>';
		
		$files = apply_filters( 'vpc_rqa_email_attachment', $files, $img_merged, $recap, $datas, $prod_id, $multiview, $total_price );
		if ( $img_merged && ! empty( $img_merged ) ) {
			return wp_mail( $recipients, $subject, $message, $headers, $files );
		}
	}

	/**
	 * Funstion to set the content type.
	 *
	 * @return string The content's type.
	 */
	public function set_content_type() {
		return 'text/html';
	}

	/**
	 * Function to add a custom post.
	 *
	 * @param  array  $datas         Datas to save.
	 * @param  int    $quantity      Product quantity.
	 * @param  string $recap         Recap of the configuration.
	 * @param  string $preview_saved The configuration's preview.
	 * @param  string $multiview     Multiview state value.
	 * @param  string $prod_id       Product ID.
	 * @return string                Post ID.
	 */
	private function add_custom_post( $datas, $quantity, $recap = '', $preview_saved = '', $multiview = '', $prod_id = '' ) {
		if ( is_array( $datas ) ) {
			foreach ( $datas as $option => $value ) {
				if ( is_array( $value ) ) {
					$value = implode( ', ', $value );
				}
				$datas[ $option ] = $value;
			}

			if ( isset( $recap['canvas_data'] ) ) {
				if ( '' !== $recap['canvas_data'] ) {
					$canvases_datas = json_decode( stripslashes( $recap['canvas_data'] ) );
					if ( isset( $canvases_datas ) && ! empty( $canvases_datas ) ) {
						foreach ( $canvases_datas as $canvas_key => $canvas ) {
							$recap = vpc_rqa_get_canvas_recap_to_save( $recap, $canvas );
						}
					}
				}
				unset( $recap['canvas_data'] );
			}

			$post_title = 'Request ' . gmdate( 'Y-m-d H:i:s' );
			$post_id    = wp_insert_post(
				array(
					'post_status' => 'publish',
					'post_type'   => 'vpc-rqa-form-data',
					'post_title'  => $post_title,
				)
			);
			update_post_meta( $post_id, 'user_datas', $datas );
			update_post_meta( $post_id, 'product_id', $prod_id );
			update_post_meta( $post_id, 'quantity', $quantity );
			if ( $recap || $preview_saved ) {
				$config_data = array(
					'recap'     => $recap,
					'preview'   => $preview_saved,
					'multiview' => $multiview,
				);
			}
			update_post_meta( $post_id, 'user_config_data', $config_data );
			return $post_id;
		}
	}

	/**
	 * Funtion to display the price container or not.
	 *
	 * @param  string $price_container_html The price container's html.
	 * @param  string $prod_id              Product ID.
	 * @return string                       The price container's html.
	 */
	public function display_price_container( $price_container_html, $prod_id ) {
		$vpc_settings = get_option( 'vpc-options' );
		$config       = $this->get_config_data( $prod_id );
		if ( get_proper_value( $config, 'request-quote', 'Yes' ) === 'Yes' && get_proper_value( $config, 'show-price', 'Yes' ) === 'No' ) {
			$price_container_html = '<div id="vpc-price-container"></div>';
		}
		return $price_container_html;
	}

	/**
	 * Function to build a form.
	 *
	 * @param  string $prod_id Product ID.
	 * @return string          The form html code.
	 */
	private function build_form( $prod_id ) {
		$vpc_options = get_option( 'vpc-options' );
		$config      = $this->get_config_data( $prod_id );
		$rqa_form    = get_proper_value( $config, 'form_selection', 'default' );
		if ( ! get_option( 'vpc-request-a-quote-add-on-license-key' ) ) {
			$html = "<h2 class='error_msg'>You have not activated your license yet. Please, activate it in order to use Request a quote add on.</h2>";
		} else {
			if ( class_exists( 'Ofb' ) && 'default' !== $rqa_form ) {
				$html = $this->create_custom_form( $rqa_form );
				$tags = array( '<form>', '</form>' );
				$html = str_replace( $tags, '', $html );
			} else {
				$html = $this->create_default_form();
			}

			$showcaptcha = get_proper_value( $vpc_options, 'rqa-captcha', 'No' );
			if ( 'Yes' === $showcaptcha ) {
				$captcha = create_captcha();
				$html   .= ' <div>'
				. '<div id="CaptachaResult" data-id="' . $captcha['result'] . '">' . $captcha['captcha'] . '</div>'
				. '<div>'
				. '<input id="captcha" name="captcha" data-validation-engine="validate[required]" type="text" value="" placeholder="' . __( 'Please solve the equation above', 'vpc-rqa' ) . '" />'
				. '</div></div>';
			}
			$html = apply_filters( 'vpc_rqa_add_custom_form', $html, $prod_id );
			$html = '<form class="formbuilt"  id="rqa_form" method="POST" enctype="multipart/form-data" action="">
					    <div class="omodal-body">' . $html . '</div>
					    <div class="omodal-footer">
					    <div class="btn">
					    <input type="submit" id="" class="submit vpc-mo-button" value="' . __( 'GET A QUOTE FOR THIS DESIGN', 'vpc-rqa' ) . '"/>
					    </div>
					    </div>
					    </form>';
		}
		return $html;
	}

	/**
	 * Function to create a custom form.
	 *
	 * @param  string $form_builder_id Form builder ID.
	 * @return string                  Custom form html code.
	 */
	private function create_custom_form( $form_builder_id ) {
		$html = display_form_builder( $form_builder_id );
		return $html;
	}

	/**
	 * Function to save preview image to the server.
	 */
	public function save_vpc_rqa_canvas_img() {
		wp_verify_nonce( 'save_vpc_rqa_canvas_img', 'rqa_canvas_img_nonce' );
		$recap         = array();
		$multiview     = '';
		$custom_vars   = '';
		$imgs_by_views = array();

		if ( isset( $_POST['recap'] ) ) {
			$sanitize_recap = vpc_rqa_sanitize_array( wp_unslash( $_POST['recap'] ) ); // phpcs:ignore
			$recap          = vpc_rqa_clean_recap( $sanitize_recap );
		}

		if ( isset( $_POST['imgs_by_views'] ) ) {
			$imgs_by_views = vpc_rqa_sanitize_array( wp_unslash( $_POST['imgs_by_views'] ) ); // phpcs:ignore
		}

		if ( isset( $_POST['multiview'] ) && ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['multiview'] ) ) ) ) {
			$multiview = sanitize_text_field( wp_unslash( $_POST['multiview'] ) );
		}

		if ( isset( $_POST['custom_vars'] ) && ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['custom_vars'] ) ) ) ) {
			$custom_vars = $_POST['custom_vars']; // phpcs:ignore
		}

		// Remove empty compenents.
		if ( isset( $recap ) && ! empty( $recap ) ) {
			foreach ( $recap as $key => $value ) {
				if ( '' === $value ) {
					unset( $recap[ $key ] );
				}
			}
		}
		$preview_url      = '';
		$merged_image_url = vpc_get_merged_image( $custom_vars, $multiview );
		$preview_url      = $merged_image_url;
		echo wp_json_encode( $preview_url );
		die();
	}


	/**
	 * Function to get the total price.
	 *
	 * @param  string $prod_id Product ID.
	 * @param  array  $recap   A configuration datas.
	 * @return string          Total price.
	 */
	public function get_total_price( $prod_id, $recap ) {
		$total_price   = 0;
		$product       = wc_get_product( $prod_id );
		$product_price = $product->get_price();
		if ( isset( $product_price ) && '' !== $product_price ) {
			$total_price += $product_price;
		}
		$config_price = VPC_Public::get_config_price( $prod_id, $recap, '', '' );
		if ( isset( $config_price ) && '' !== $config_price ) {
			$total_price += $config_price;
		}
		$total_price = wc_price( $total_price );
		return $total_price;
	}

}
